<?php
/**
 * Plugin PDF_VERSION pour Spip 3.x
 * Licence GPL (c) 2016 Cedric
 *
 */

if (!defined("_ECRIRE_INC_VERSION")) return;


/**
 * Appel de PDFShift  par API sur un autre site SPIP
 * @param string $html_file
 * @param string $pdf_file
 * @param array $args
 */
function pdf_version_services_pdfshift_dist($html_file, $pdf_file, $args = array()) {

	include_spip('inc/config');
	$logfile = 'pdfv_pdfshift';

	if($pdfshift_api_key = lire_config('pdf_version/pdfshift_api_key')) {

		$pdfshift_api_url = "https://api.pdfshift.io/v3/convert/pdf";

		// Appel de pdfshift via API http

		$html = "";
		$pdfshift_options = [
			'source' => '(...)',
			'use_print' => false,
			'delay' => 500,
			'format' => $args['--page-size'] ?? 'A4',
			'landscape' => empty($args['--orientation']) ? false : ($args['--orientation'] === 'Portrait' ? false : true),
			'margin' => implode(' ', [$args['--margin-top'], $args['--margin-right'], $args['--margin-bottom'], $args['--margin-left']]),
		];

		// margin et format semblent pas vraiment pris en compte, on doublonne avec une directive css
		$pdfshift_options['css'] = "@page{size:".$pdfshift_options['format'].";margin:".$pdfshift_options['margin']."}";

		// todo : substitution des variables dans le header et footer
		/*
			Variable 	Description
			{{date}} 	Formatted print date
			{{title}} 	Title of the HTML document
			{{url}} 	Page URL
			{{page}} 	Current page
			{{total}} 	Total number of pages
		*/
		$vars = [
			'@page@' => '{{page}}',
			'@total_page@' => '{{total}}',
		];

		if (isset($args['--header-html'])) {
			lire_fichier($args['--header-html'], $html);
			$html = pdf_version_renomme_variables($vars, $html);

			// il faut reinserer les margin left/right du document dans le footer car il est par defaut sans margin
			$mleft = $args['--margin-left'];
			$mright = $args['--margin-right'];
			$style_add = "<style>#header {margin-left:calc(0.75 * {$mleft});margin-right:calc(0.75 * {$mright});}</style>";
			if ($p = strpos($html, "</head>")) {
				$html = substr_replace($html, $style_add, $p, 0);
			}
			else {
				$html .= $style_add;
			}
			$pdfshift_options['header'] = ['source' => $html, 'height' => $args['--header-spacing'] ?? '1px'];
			if (is_numeric($pdfshift_options['header']['height'])) {
				$pdfshift_options['header']['height'] .= 'px';
			}
		}
		if (isset($args['--footer-html'])) {
			lire_fichier($args['--footer-html'], $html);
			$html = pdf_version_renomme_variables($vars, $html);

			// il faut reinserer les margin left/right du document dans le footer car il est par defaut sans margin
			$mleft = $args['--margin-left'];
			$mright = $args['--margin-right'];
			$mbottom = $args['--margin-bottom'];
			$style_add = "<style>#footer {margin-left:calc(0.75 * {$mleft});margin-right:calc(0.75 * {$mright});margin-bottom: calc(0.5 * $mbottom)}</style>";
			if ($p = strpos($html, "</head>")) {
				$html = substr_replace($html, $style_add, $p, 0);
			}
			else {
				$html .= $style_add;
			}
			$pdfshift_options['footer'] = ['source' => $html, 'height' => $args['--footer-spacing'] ?? '1px'];
			if (is_numeric($pdfshift_options['footer']['height'])) {
				$pdfshift_options['footer']['height'] .= 'px';
			}
		}

		if (lire_config('pdf_version/pdfshift_sandbox')) {
			$pdfshift_options['sandbox'] = true;
		}

		spip_log($pdfshift_api_url . " $html_file -> $pdf_file : options=" . json_encode($pdfshift_options), $logfile . _LOG_DEBUG);

		// lire le html et le renseigner dans les arguments
		$html = "";
		lire_fichier($html_file, $html);
		$pdfshift_options['source'] = $html;


		// un watermark ?
		$watermarks = [
			'pdf' => [
				'--pdfshit-watermark-source'
			],
			'texte' => [
				'--pdfshit-watermark-text',
				'--pdfshit-watermark-font_size',
				'--pdfshit-watermark-font_family',
				'--pdfshit-watermark-font_color',
				'--pdfshit-watermark-font_opacity',
				'--pdfshit-watermark-font_bold',
				'--pdfshit-watermark-font_italic',
				'--pdfshit-watermark-offset_x',
				'--pdfshit-watermark-offset_y',
				'--pdfshit-watermark-rotate'
			],
			'image' => [
				'--pdfshit-watermark-image',
				'--pdfshit-watermark-offset_x',
				'--pdfshit-watermark-offset_y',
				'--pdfshit-watermark-rotate'
			],
		];
		foreach ($watermarks as $type => $vars) {
			$watermark = pdf_version_collecter_variables_head($vars, $html);
			if (!empty($watermark)) {
				$pdfshift_options['watermark'] = [];
				foreach ($watermark as $k => $v) {
					$k = substr($k, strlen('--pdfshit-watermark-'));
					$pdfshift_options['watermark'][$k] = $v;
				}
				break;
			}
		}

		$result = true;
		$curl = curl_init();
		curl_setopt_array($curl, array(
		    CURLOPT_URL => $pdfshift_api_url,
		    CURLOPT_RETURNTRANSFER => true,
		    CURLOPT_POST => true,
		    CURLOPT_POSTFIELDS => json_encode($pdfshift_options),
		    CURLOPT_HTTPHEADER => array('Content-Type:application/json'),
		    CURLOPT_USERPWD => "api:$pdfshift_api_key"
		));

		$response = curl_exec($curl);
		$res = curl_getinfo($curl);
		$trace = json_encode($res);

		if (!empty($res['http_code'])
			and $res['http_code'] === 200
			and $response !== false) {
			file_put_contents("{$pdf_file}.tmp", $response);
			$response = substr($response, 0, 16000);
			$response = ltrim($response);
			if (stripos($response, '<!DOCTYPE html>')===0
				and stripos($response, "<html")!==false
				and stripos($response, "minipres")!==false){
				// c'est visiblement une minipres html, donc le serveur a foire et on a pas reconnus le status code :(
				@unlink("{$pdf_file}.tmp");
				$result = false;

				// renseigner le status pour les logs au lieu du 200
				if (preg_match(",<title>(.*)</title>,Uims", $response, $m)){
					$res['http_code'] .= " / " . $m[1];
				}
			}
		}
		else {
			$result = false;
			// en cas d'erreur pdfshift renvoie un json avec le code d'erreur et le message, on le loge donc
			if ($response !== false and strlen($response)) {
				$trace .= " | " . $response;
			}
		}

		if ($result) {
			@rename("{$pdf_file}.tmp", $pdf_file);
		}
		else {
			@unlink("{$pdf_file}.tmp");
		}

		spip_log($pdfshift_api_url . " $html_file -> $pdf_file : $trace", $logfile . ($result ? '' : _LOG_ERREUR));

		return $result;
	}

	spip_log("service pdfshift mal configure",$logfile . _LOG_ERREUR);
	return false;
}
